// Lawson Labs, Inc.
// 15 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800             August 30, 1996  version 1

#define STRICT
#include <windows.h>
#include <stdio.h>
#include <alloc.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <dos.h>
#ifdef __cplusplus
	 #define __CPPARGS ...
#else
	 #define __CPPARGS
#endif

//typedef for M141 struct
typedef struct{
unsigned int	adc141;           //ADC for Model 141
double	      hertzRate;			//scan rate (hz)
unsigned int	average;          //number of samples to average per point
unsigned int   ichDelay;         //interchannel delay
unsigned int   numData;          //number of datapoints per scan
unsigned long	totScans;         //number of scans to perform
unsigned int   digInp;           //flag for digital input 1 = on  0 = off
unsigned int   biPolar;				//1 = bipolar, 0 = unipolar
BYTE 	         controlCode;		//control code for next conversion in GetConv
unsigned int   cCode[33];        //array of control codes
unsigned long	numCompleted;     //number of scans completed if scan was cut short
DWORD          calTime;			   //time in milliseconds for calibration
}	T_M141;

//local prototypes
void         SetWindowToDesktop(void);
short        CheckADC(unsigned int adc141);
void         PrintCommandMenu(T_M141 far *M141);
void         DoCommand(T_M141 far *M141, char ch, double *max, double *min);
void         StandardDev(double *stDv,
								 double *rmsBits,
								 double *sdTot,
								 double *runningAvg,
								 double *sd,
								 double *voltage,
								 int    *sdCount,
								 int		biPolar,
								 int 		sdGrp);
unsigned int GetAvg(unsigned int average);
void         SaveData(T_M141 far *M141, unsigned long huge *dataBuffer);
void         DispCodes(T_M141 far *M141);
short        SetupScan(T_M141 far *M141, double *maxHertzRate);
short        ReadConfig(T_M141 far *M141);
short        WriteConfig(T_M141 far *M141);
void 			 WaitSettle(T_M141 far *M141);	//wait for channel to settle
short			 GetADC(T_M141 far *M141);			//get and check ADC address

//prototypes for external DLL functions
short  (FAR PASCAL *ADInit)(int adc); //test for existence of 141 at ADC address
long   (FAR PASCAL *GetCount)(T_M141 far *M141);  //get one conversion in counts and digital input status
double (FAR PASCAL *GetConv)(T_M141 far *M141);   //get one conversion in volts and digital input status
short  (FAR PASCAL *Call141)(T_M141 far *M141,    //get conversions based on the T_M141 structure
									  unsigned long huge *clientDataBuffer);
short  (FAR PASCAL *Calibrate)(T_M141 far *M141); //calibrate 141

//constants
const double tickTime = .001; //time in seconds for each tick (GetTickCount resolution)

//Global Variables
HINSTANCE          hLibrary;

int main(void)
{
T_M141             M141;
unsigned long huge *StoreConv;		//pointer for storage of conversions
HANDLE             hglbStoreConv;
double             scanTime, maxHertzRate, voltage, max=-6000, min=6000;
unsigned long      arraySize;        //size of storage area in bytes
long               i, j;
int					 n, p_row, p_col, incFlag, endDisplay, cStat;
short					 calFlag = 0;
char               tmpbuf[80], ch;
DWORD					 tckTime;
//variables for standard deviation calculation
double             stDv=0, rmsBits=0, sdTot = 0, runningAvg=0, sd[201];;
int                sdCount=1, sdGrp=40;

	hglbStoreConv = NULL;				//no handle allocated for storage array
	SetWindowToDesktop();

	hLibrary = LoadLibrary ("141DLL.DLL");  //get handle for DLL
	//get pointers to DLL functions
	(FARPROC) ADInit =GetProcAddress(hLibrary, "ADInit");
	(FARPROC) GetCount =GetProcAddress(hLibrary, "GetCount");
	(FARPROC) GetConv =GetProcAddress(hLibrary, "GetConv");
	(FARPROC) Call141 =GetProcAddress(hLibrary, "Call141");
	(FARPROC) Calibrate =GetProcAddress(hLibrary, "Calibrate");

	if (!ReadConfig(&M141))
	{
	  gotoxy(24,10);
	  printf("unable to read 141EWIN.CFG file");
	  gotoxy(26,11);
	  printf("press any key to continue");
	  getch();
	}

TOP_MENU:
  if (hglbStoreConv != NULL)				//if handle to storage array exists,
  {                                    //free handle
	 GlobalUnlock(hglbStoreConv);
	 GlobalFree(hglbStoreConv);
  }

  ch = 0;

  while(ch != 27)
  {
	 clrscr();
	 gotoxy(27,4);
	 printf ("MODEL 141");
	 gotoxy(27,6);
	 printf ("Main Menu");
	 gotoxy(20,8);
	 printf ("1) ADC              %u",M141.adc141);
	 gotoxy(20,10);
	 printf ("2) Polled Mode Operation");
	 gotoxy(20,12);
	 printf ("3) Scanning Mode Operation");
	 gotoxy(18,14);
	 printf ("Esc) Exit program");

	 ch = getch();

	 if (ch == 27)
	 {
		goto EXIT_MAIN;
	 }

	 if (ch == '1')
	 {
		calFlag = GetADC(&M141);  //get and check ADC address
	 }

	 if (ch == '2')
	 {

		if (calFlag == 0)	//if not calibrated, ADC address from config file not verified
		{

		  if (ADInit(M141.adc141)==0)		//check for 141 at ADC address
		  {
			 clrscr();
			 printf ("\nModel 141 not found at ADC %u\n",M141.adc141);
			 printf("press any key to continue");
			 getch();
			 goto TOP_MENU;
		  }

		  gotoxy(1,24);
		  printf("                           Calibrating...                       ");
		  if (Calibrate(&M141)==0)
		  {
			 clrscr();
			 printf ("\nCalibration was not successful\n");
			 printf("press any key to continue");
			 getch();
			 goto TOP_MENU;
		  }
		  calFlag = 1;
		}

		sdCount=1;					//initialize standard deviation count
		goto POLLED;
	 }

	 if (ch == '3')
	 {

		if (calFlag == 0)	//if not calibrated, ADC address from config file not verified
		{

		  if (ADInit(M141.adc141)==0)		//check for 141 at ADC address
		  {
			 clrscr();
			 printf ("\nModel 141 not found at ADC %u\n",M141.adc141);
			 printf("press any key to continue");
			 getch();
			 goto TOP_MENU;
		  }

		  gotoxy(1,24);
		  printf("                           Calibrating...                       ");
		  if (Calibrate(&M141)==0)
		  {
			 clrscr();
			 printf ("\nCalibration was not successful\n");
			 printf("press any key to continue");
			 getch();
			 goto TOP_MENU;
		  }
		  calFlag = 1;
		}

		goto GET_SCANS;
	 }

  } //while Ch != 27

  goto EXIT_MAIN;

POLLED:		//polled mode example

  WaitSettle(&M141);

  ch = 0;
  p_row = 13;

  clrscr();
  PrintCommandMenu(&M141);
  while (ch != 27)              //repeat until esc key pressed or an error
  {

	 while (!kbhit())//loop until keyboard hit
	 {

		voltage = GetConv(&M141)* 1000;
		gotoxy(2,p_row++);

		printf ("  %10.3lf      ",voltage);

		if (max < voltage)
		{
		  max = voltage;               //update minimum
		}

		if (min > voltage)
		{
		  min = voltage;               //update minimum
		}


		if (M141.calTime >= 5000)
		{
		  sdGrp = 40;
		}
		else if ((M141.calTime > 1500) && (M141.calTime < 5000))
		{
		  sdGrp = 100;
		}
		else if (M141.calTime <= 1500)
		{
		  sdGrp = 200;
		}

		StandardDev(&stDv,
						&rmsBits,
						&sdTot,
						&runningAvg,
						sd,
						&voltage,
						&sdCount,
						M141.biPolar,
						sdGrp);

		gotoxy (57,5);
		if (M141.digInp == 1)
		{
		  printf("on ");
		}
		else
		{
		  printf("off");
		}

		if (p_row == 23)
		{
		  p_row = 13;

		  gotoxy(40,14);

		  printf ("Maximum     %8.3lf mv       ", max);
		  gotoxy (40,15);
		  printf ("Minimum     %8.3lf mv       ", min);
		  gotoxy (40,16);
		  printf ("Delta       %8.3lf mv       ", fabs(max - min));
		  gotoxy (40,17);
		  printf ("Average     %8.3lf mv       ", runningAvg);
		  gotoxy (40,19);
		  printf ("Standard deviation  %8.3lf mv   ", stDv);
		  gotoxy (40,20);
		  printf ("Effective Resolution  %5.1lf  bits", rmsBits);
		}

	 }   //while (!kbhit())

	 ch = getch();

	 if (ch != 27)
	 {

		if (islower(ch))
		{
		  ch = toupper(ch);
		}

		DoCommand (&M141, ch, &max, &min);
	 }

  }     //while (Ch != 27)

  goto TOP_MENU;  //end of polled mode example

GET_SCANS:    		//scanning mode example

  if (hglbStoreConv != NULL)				//if handle to storage array exists,
  {                                    //free handle
	 GlobalUnlock(hglbStoreConv);
	 GlobalFree(hglbStoreConv);
  }

	if (!SetupScan(&M141, &maxHertzRate))
	{
	  goto TOP_MENU;
	}

GET_NUM_SCANS:
  clrscr();
  printf("%lu scans per sampling run (Y/N)? ",M141.totScans);
  ch = getch();

  if (ch == 27)
  {
	 goto TOP_MENU;
  }

  if (ch == 'N' || ch == 'n')
  {
	 printf("\n\nEnter scans per sampling run ");
	 fgets(tmpbuf,80,stdin);
	 sscanf(tmpbuf,"%lu",&M141.totScans);

	 if (M141.totScans == 0)
	 {
		M141.totScans = 1;
	 }

	 goto GET_NUM_SCANS;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_NUM_SCANS;
  }

  scanTime = (1/M141.hertzRate)*(M141.totScans-1) + 1.0/maxHertzRate;

  arraySize = (M141.totScans*M141.numData)*4;  //each datapoint is 4 bytes

  hglbStoreConv=GlobalAlloc(GPTR,arraySize);
  if (hglbStoreConv == NULL)
  {
	 printf("\n\nunable to allocate %ld bytes. Reduce number of scans\n",arraySize);
	 printf("any key to continue");
	 getch();
	 goto GET_NUM_SCANS;
  }

  StoreConv = (unsigned long huge *)GlobalLock(hglbStoreConv);  //allocate storage area

DISPLAY:
	clrscr();

	printf("\nData points per scan = %u\n", M141.numData);
	printf("Average %u samples per data point\n",M141.average);
	printf("Inter-channel delay = %u  milliseconds\n",M141.ichDelay);
	printf("Scans per sample run = %lu\n", M141.totScans);
	printf("Size of array = %lu bytes\n",arraySize);
	printf("Scanning rate = %8.2lf hz\n",M141.hertzRate);
	printf("Sample run duration %8.2lf seconds\n",scanTime);
	printf("\n\nIs everything correct (Y/N)? ");
	ch = getch();

	if (ch == 27)
	{
	  goto TOP_MENU;
	}

	if (ch == 'N' || ch == 'n')
	{
	  goto GET_SCANS;
	}
	else if (ch != 'Y' && ch != 'y')
	{
	  goto DISPLAY;
	}

REVIEW_CODES:
	clrscr();
	printf ("Review or revise control codes (Y/N)? ");
	ch = getch();


	if (ch == 27)
	{
	  goto TOP_MENU;
	}

REVIEW_CODES2:

	if (ch == 'Y' || ch == 'y')
	{

	  DispCodes(&M141);

	  p_row = 4;				//initialize starting row and col
	  p_col = 1;
	  incFlag = 0;				//not incrementing
	  M141.cCode[0] = -1;	//initialize cCode[0] for increment

	  for (n = 1;n <= M141.numData;n++)
	  {
		 gotoxy(p_col,p_row++);

		 if (incFlag == 0)				//if not incrementing
		 {

			if (M141.cCode[n] < 10)
			{
			  gotoxy(p_col+12,p_row-1);
			}
			else
			{
			  gotoxy(p_col+11,p_row-1);
			}

			fgets(tmpbuf,80,stdin);		//get control code for current channel

			if (tmpbuf[0] == 'i' || tmpbuf[0] == 'I') //if entered 'i' then start incrementing
			{
			  incFlag = 1;
			  M141.cCode[n] = M141.cCode[n-1]+1;	//increment control code
			  if (M141.cCode[n] >= 32)
			  {
				 M141.cCode[n] = 0;
			  }
			  gotoxy(p_col+11,p_row-1);
			  printf ("%2u  ",M141.cCode[n]);
			}
			else
			{
			  gotoxy(p_col+12,p_row-1);
			  printf(" ");
			  gotoxy(p_col+12,p_row-1);

			  sscanf(tmpbuf,"%u",&M141.cCode[n]);	//store current control code

			  if (M141.cCode[n] >= 32)	 //control codes must be >= 0 and < 32
			  {
				 M141.cCode[n] = 31;
			  }

			}

		 } //if incFlag == 0
		 else if (incFlag == 1)
		 {
			M141.cCode[n] = M141.cCode[n-1]+1;
			if (M141.cCode[n] >= 32)	//control code rolls over at 32
			{
			  M141.cCode[n] = 0;
			}
		 }

		 gotoxy(p_col+11,p_row-1);
		 if ((p_col + 20) > 60)   		//keep display in window
		 {
			printf ("%2u    ",M141.cCode[n]);
		 }
		 else
		 {
			printf ("%2u       ",M141.cCode[n]);
		 }

		 if (p_row == 20 && (n != M141.numData))  //keep display in window
		 {
			p_col += 19;
			p_row = 4;
		 }

	  }  //end for loop

	  M141.cCode[n] = M141.cCode[1];

EXIT_CODES:
	  gotoxy(10,24);
	  printf ("Revise control codes (Y/N)?                                     ");
	  gotoxy(38,24);
	  ch = getch();

	  if (ch == 27)
	  {
		 goto TOP_MENU;
	  }

	  if (ch == 'Y' || ch == 'y')
	  {
		 goto REVIEW_CODES2;
	  }
	  else if (ch != 'N' && ch != 'n')
	  {
		 goto EXIT_CODES;
	  }

	}
	else if (ch != 'N' && ch != 'n')
	{
	  goto REVIEW_CODES;
	}

	outp(M141.adc141, M141.cCode[1]+M141.biPolar);	//select first channel
	WaitSettle(&M141);

	clrscr();
	printf("\nSampling will take %8.2lf seconds",scanTime);
	printf("\npress a key to start scanning, ESC to exit");

	if (getch() == 27)
	{
	  goto TOP_MENU;
	}

	gotoxy(1,3);
	printf("Activating digital input will cut scan short\n");
	gotoxy(1,4);

	tckTime = GetTickCount(); 					//get start time in milliseconds
	cStat = Call141(&M141, StoreConv);
	tckTime = GetTickCount() - tckTime;    //get elapsed time in milliseconds
	if (cStat == 0)
	{
	  printf ("\nUnable to find Model 141\n");
	  printf("press any key to continue");
	  getch();
	  goto TOP_MENU;
	}
	else if(cStat == 2)
	{
	  printf ("Sampling stopped. ");
	}

	printf ("%lu of %lu scans completed\n",M141.numCompleted, M141.totScans);

	if(cStat == 1)
	{
	  printf ("\nSampling complete.\n");
	}
	printf ("Elapsed time:   %8.2lf",tckTime/1000.0);

SAVE_FILE:
	gotoxy(1,8);
	printf("Do you want to save the data as a text file (Y/N)?     ");
	gotoxy(52,8);
	ch = getch();

	if (ch == 'Y' || ch == 'y')
	{
	  SaveData(&M141, StoreConv);			//save data to disk
	}
	else if (ch != 'N' && ch != 'n')
	{
	  goto SAVE_FILE;
	}


DISP_SCANS:
	endDisplay =  ceil((double)(M141.numData)/8.0);	//formatting to keep display in window
	clrscr();
	gotoxy(1,25);
	printf("any key to continue, ESC to end                    ");
	gotoxy(1,1);

	for (i = 0; i < M141.numCompleted; i++)
	{
	  printf("%-4ld", i+1);

	  for (j = 0; j < (M141.numData); j++)
	  {

		 if (M141.biPolar)		//calculate voltage for bipolar or unipolar
		 {
			voltage = ((double)StoreConv[i*M141.numData + j])*9.536743e-3 - 5000;
		 }
		 else
		 {
			voltage = ((double)StoreConv[i*M141.numData + j])*4.768372e-3;
		 }

		 printf (" %8.3lf ",voltage);


		 if (wherex() > 70)				//keep display in window
		 {
			printf("\n    ");
		 }
	  }

	  printf ("\n");
	  if ((endDisplay + wherey()) > 24)		//pause for user input
	  {
		 if (getch() == 27)
		 {
			goto END_SCANS;
		 }
		 gotoxy(1,1);
	  }
	}

END_SCANS:
	gotoxy(1,25);
	printf("R to retry, ESC for menu                              ");
	gotoxy(26,25);

RETRY:
	ch = getch();

	if (ch == 'R' || ch == 'r')
	{
	  goto DISPLAY;
	}
	else if( ch == 27)
	{
	  goto TOP_MENU;
	}

	goto RETRY;

EXIT_MAIN:
	FreeLibrary(hLibrary);			//free DLL

SAVE_CONFIG:
	clrscr();
	printf("\nDo you wish to save configuration (Y/N)? ");
	ch = getch();

	if (ch == 'Y' || ch == 'y')
	{
	  WriteConfig(&M141);
	}
	else if(ch != 'N' && ch != 'n')
	{
	  goto SAVE_CONFIG;
	}

	DestroyWindow(GetActiveWindow());			//close window
	exit(0);
	return 1;
}

void SetWindowToDesktop(void)  //make window the same size as the desktop
{
RECT rect;

  clrscr();
  GetWindowRect(GetDesktopWindow(), &rect);
  SetWindowPos(GetActiveWindow(), (HWND) NULL, 0, 0,
					rect.right, rect.bottom,
					SWP_NOZORDER | SWP_NOACTIVATE);
}

short GetADC(T_M141 far *M141)		//get and check ADC address
{
char  tmpbuf[80];

GET_ADC:
  clrscr();
  printf("Enter ADC (check your manual) ");
  fgets(tmpbuf,80,stdin);
  sscanf(tmpbuf,"%u",&M141->adc141);

  if (!CheckADC(M141->adc141))
  {
	 printf ("\n%u is not a valid ADC address \n",M141->adc141);
	 printf("press ENTER to try again");

	 if (getch() == 27)
	 {
		return 0;
	 }
	 goto GET_ADC;

  }

  if (ADInit(M141->adc141)==0)
  {
	 printf ("\nModel 141 not found at ADC %u\n",M141->adc141);
	 printf("press ENTER to try again, ESC to abort");

	 if (getch() == 27)
	 {
		return 0;
	 }

	 goto GET_ADC;
  }

  gotoxy(1,24);
  printf("                           Calibrating...                       ");
  if (Calibrate(M141)==0)
  {
	 clrscr();
	 printf ("\nCalibration was not successful\n");
	 printf("press any key to continue");
	 getch();
	 return 0;
  }
  return 1;
}

void PrintCommandMenu(T_M141 far *M141)	//print polled mode command menu
{

  gotoxy (15,1);
  printf ("POLLED  MENU   Lawson Labs Model 141");
  gotoxy (5,3);
  printf ("C) A/D input Channel    %u", (M141->controlCode>>4));
  gotoxy (5,4);
  printf ("E) External sub-channel %u",M141->controlCode&0xF);

  if ((M141->controlCode&0xF) < 10)
  {
	 printf (" ");
  }

  gotoxy (5,5);
  printf ("P) ");

  if (M141->biPolar == 0x80)
  {
	 printf("biPolar ");
  }
  else
  {
	 printf("uniPolar");
  }

  gotoxy (40,22);
  printf ("A/D clock ");
  if (M141->calTime >= 5000)
  {
	 printf("224 KHz  ");
  }
  else if ((M141->calTime > 1500) && (M141->calTime < 5000))
  {
	 printf("895 KHz  ");
  }
  else if (M141->calTime <= 1500)
  {
	 printf("3.58 MHz  ");
  }

  gotoxy (43,3);
  printf ("A) Averaging            %-5u", M141->average);

  gotoxy (43,4);
  printf ("R) Recalibrate");

  gotoxy (43,5); printf ("Digital input ");
  if (M141->digInp == 1)
  {
	 printf("on ");
  }
  else
  {
	 printf("off");
  }

  gotoxy (6,11); printf ("Millivolts\n");
  gotoxy (1,24);
  printf (" Press ESC for menu                                              ");
}

void DoCommand(T_M141 far *M141, char ch, double *max, double *min)
{
int     tempExt;
char    tmpbuf[80];

  if (ch == 'A')
  {
	 M141->average = GetAvg(M141->average);		//get averaging
  }

  if (ch == 'C')											//toggle channel
  {

	 if (M141->controlCode > 15)
	 {
		M141->controlCode = M141->controlCode&0xF;
	 }
	 else
	 {
		M141->controlCode = M141->controlCode + 0x10;
	 }
	 outp(M141->adc141, M141->controlCode+M141->biPolar);	//select current channel
	 WaitSettle(M141);
  }

  if (ch == 'E')
  {
	 tempExt = -1;
	 gotoxy(1,24);
	 printf("                                              ");

	 while ((tempExt < 0) || (tempExt > 15))
	 {
		gotoxy(1,24);
		printf("Enter sub channel                           ");
		gotoxy(19,24);
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",&tempExt);
	 }

	 M141->controlCode = ((M141->controlCode>>4)<<4) + tempExt;

	 outp(M141->adc141, M141->controlCode+M141->biPolar);	//select current channel
	 WaitSettle(M141);
  }

  if (ch == ' ')
  {
	 (*max) = -6000.0;
	 (*min) = 6000.0;   //reset max-min
  }

  if (ch == 'R')
  {
	 gotoxy(1,24);
	 printf("                           Calibrating...                       ");
	 if (Calibrate(M141)==0)
	 {
		clrscr();
		printf ("\nCalibration was not successful\n");
		printf("press any key to continue");
		getch();
	 }
	 WaitSettle(M141);
  }

  if (ch == 'P')										//toggle polarity
  {

	 if (M141->biPolar == 0)
	 {
		M141->biPolar = 0x80;
	 }
	 else
	 {
		M141->biPolar = 0;
	 }

	 gotoxy(1,24);
	 printf("                           Calibrating...                       ");
	 if (Calibrate(M141)==0)
	 {
		clrscr();
		printf ("\nCalibration was not successful\n");
		printf("press any key to continue");
		getch();
	 }
	 outp(M141->adc141, M141->controlCode+M141->biPolar);	//select current channel
	 WaitSettle(M141);
  }

  PrintCommandMenu(M141);
}

unsigned int GetAvg(unsigned int average)		//get averaging (1,2,4,8...256)
{
int tempCh;

  tempCh = 0;

  while((tempCh != 27) && (tempCh != 13))
  {

	 while (!kbhit())
	 {
		gotoxy (1,24);
		printf ( "Press + increase, - decrease averaging, Enter to continue ");
		printf ( "%-6u", average);
	 }

	 tempCh =(short) getch();
	 if (tempCh == '+')
	 {
		average *= 2;
		if (average == 0)
		{
		  average = 1;
		}
	 }

	 if (tempCh == '-')
	 {
		average /= 2;
	 }

	 if (average  < 1)
	 {
		average = 256;
	 }
	 else if (average > 256)
	 {
		average = 1;
	 }

  }
  return average;
}

void SaveData(T_M141 far *M141, unsigned long huge *dataBuffer)
{
char             tmpbuf[80], textFile[13];
long             i, j;
double           voltage;
FILE             *tFile;  //handle for saving text file to disk

  printf("\n\nEnter the name of the text file ");
  fgets(tmpbuf,80,stdin);
  sscanf(tmpbuf,"%13s",&textFile);

  if(strlen(tmpbuf))
  {
	 if ((tFile = fopen(textFile, "wt")) != NULL)
	 {
		printf("Saving data...");
		for (i = 0; i < (M141->numCompleted); i++)
		{
		  fprintf(tFile,"%-5ld ", i+1);
		  for (j = 0; j < (M141->numData); j++)
		  {

			 if (M141->biPolar)
			 {
				voltage = ((double)dataBuffer[i*M141->numData + j])*9.536743e-3 - 5000;
			 }
			 else
			 {
				voltage = ((double)dataBuffer[i*M141->numData + j])*4.768372e-3;
			 }

			 fprintf (tFile," %8.3lf",voltage);

		  }
		  fprintf (tFile,"\n");
		}
		fclose(tFile);
		gotoxy(1,wherey());
		printf("Data saved in file %s",textFile);
	 }
	 else
	 {
		printf("unable to open file %s\nPress any key to continue",textFile);
		getch();
	 }
  }
}

void DispCodes(T_M141 far *M141)			//display control codes
{
int p_row, p_col, n;

  clrscr();
  gotoxy(1,1);
  printf("  data\n");
  printf("  point   CCode\n");
  gotoxy(10,24);
  printf("Press RETURN to leave unchanged");

  if (M141->numData > 1)
  {
	 printf(", I to increment ");
  }

  p_row = 4;
  p_col = 1;
  for (n = 1;n <= M141->numData;n++)
  {
	 gotoxy(p_col,p_row++);
	 printf("   %2d      %2u  ", n, M141->cCode[n]);
	 if (p_row == 20 && (n != M141->numData))
	 {
		p_col += 19;
		p_row = 4;
		gotoxy(p_col, 1);
		printf("  data");
		gotoxy(p_col, 2);
		printf("  point   CCode");
	 }

  }
  M141->cCode[n] = M141->cCode[1];
}

short ReadConfig(T_M141 far *M141)
{
int n;
char tmpbuf[80];
FILE *rCfg;

	if ((rCfg = fopen("141ewin.cfg", "rt")) != NULL)
	{
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M141->adc141);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%lf",&M141->hertzRate);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M141->average);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M141->ichDelay);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M141->numData);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%lu",&M141->totScans);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M141->biPolar);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%u",&M141->controlCode);
	  for (n = 0;n<= 33;n++)
	  {
		 fgets(tmpbuf,80,rCfg);
		 sscanf(tmpbuf,"%u",&M141->cCode[n]);
	  }

	  fclose(rCfg);
	  return 1;
	}
	else
	{
	  M141->adc141 = 560;
	  M141->hertzRate = 1000;
	  M141->ichDelay = 568;			//assume settling time for 874 Hz rate
	  M141->average = 1;
	  M141->numData = 1;
	  M141->totScans = 100;
	  M141->controlCode = 0;
	  M141->biPolar = 0x80;			//default bipolar
	  for (n=0;n<=33;n++)
	  {
		 M141->cCode[n] = 0;
	  }

	  return 0;
	}

}


short WriteConfig(T_M141 far *M141)
{
int n;
FILE *wCfg;

  if ((wCfg = fopen("141ewin.cfg", "wt")) != NULL)
  {
	 fprintf(wCfg,"%u\n",M141->adc141);
	 fprintf(wCfg,"%lf\n",M141->hertzRate);
	 fprintf(wCfg,"%u\n",M141->average);
	 fprintf(wCfg,"%u\n",M141->ichDelay);
	 fprintf(wCfg,"%u\n",M141->numData);
	 fprintf(wCfg,"%lu\n",M141->totScans);
	 fprintf(wCfg,"%u\n",M141->biPolar);
	 fprintf(wCfg,"%u\n",M141->controlCode);
	 for (n = 0;n<= 33;n++)
	 {
		fprintf(wCfg,"%u\n",M141->cCode[n]);
	 }
	 fclose(wCfg);
	 return 1;
  }

  return 0;
}

short        SetupScan(T_M141 far *M141,
							  double *maxHertzRate)
{
char ch, tmpbuf[80];
double updateTime;

GET_POINTS:
  clrscr();
  printf("data points/scan= %u (Y/N)? ",M141->numData);
  ch = getch();

  if (ch == 27)
  {
	 return 0;
  }

  if (ch == 'N' || ch == 'n')
  {
	 printf("\n\nEnter number of data points per scan ");
	 fgets(tmpbuf,80,stdin);
	 sscanf(tmpbuf,"%u",&M141->numData);

	 if (M141->numData == 0)
	 {
		M141->numData = 1;
	 }
	 else if (M141->numData > 32)
	 {
		M141->numData = 32;
	 }

	 goto GET_POINTS;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_POINTS;
  }

GET_AVG:
  clrscr();
  printf("average %u samples/data point (Y/N)? ",M141->average);
  ch = getch();

  if (ch == 27)
  {
	 return 0;
  }

  if (ch == 'N' || ch == 'n')
  {
	 M141->average = GetAvg(M141->average);
	 goto GET_AVG;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_AVG;
  }

  if (M141->numData == 1)
  {
	 M141->ichDelay = 0;
  }
  else if (M141->numData > 1)
  {

GET_ICH:
	 clrscr();
	 gotoxy(1,10);
	 printf("minimum settling time = ");

	 if (M141->calTime >= 5000)
	 {
		printf("2270 milliseconds");
	 }
	 else if ((M141->calTime > 1500) && (M141->calTime < 5000))
	 {
		printf("568 milliseconds");
	 }
	 else if (M141->calTime <= 1500)
	 {
		printf("142 milliseconds");
	 }

	 gotoxy(1,2);
	 printf("interchannel delay =  %u  milliseconds (Y/N)? ",M141->ichDelay);
	 ch = getch();

	 if (ch == 27)
	 {
		return 0;
	 }

	 if (ch == 'N' || ch == 'n')
	 {
		printf("\n\nEnter Interchannel delay count ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%u",&M141->ichDelay);

		goto GET_ICH;
	 }
	 else if (ch != 'Y' && ch != 'y')
	 {
		goto GET_ICH;
	 }
  }

GET_RATE:
	clrscr();

  if (M141->calTime >= 5000)
  {
	 updateTime = 1.0/218.0;
  }
  else if ((M141->calTime > 1500) && (M141->calTime < 5000))
  {
	 updateTime = 1.0/874.0;
  }
  else if (M141->calTime <= 1500)
  {
	 updateTime = 1.0/3500.0;
  }

  if (M141->numData == 1)
  {
	 *maxHertzRate = 1.0/(updateTime * M141->average * M141->numData);
  }
  else
  {
	 *maxHertzRate = 1.0/((updateTime*M141->average+((double)M141->ichDelay/1000.0) + .0003) * M141->numData);
  }

  if (*maxHertzRate > 1000)
  {
	 *maxHertzRate = 1000;
  }

  if (*maxHertzRate < 0)
  {
	 *maxHertzRate = 1;
  }

  if ((M141->hertzRate+.005) > *maxHertzRate)
  {
	 printf("%8.2lf Max Rate (Y/N)? ",*maxHertzRate);
  }
  else
  {
	 printf("%8.2lf scan rate (Y/N)? ",M141->hertzRate);
  }

  ch = getch();

  if (ch == 27)
  {
	 return 0;
  }

  if (ch == 'N' || ch == 'n')
  {
	 printf ("\n\nMax Rate %8.2lf",*maxHertzRate);
	 printf("\n\nEnter scan rate ");
	 fgets(tmpbuf,80,stdin);
	 sscanf(tmpbuf,"%lf",&M141->hertzRate);

	 if (M141->hertzRate <= 0)
	 {
		M141->hertzRate = 1;
	 }

	 if (M141->hertzRate < 0)
	 {
		M141->hertzRate = 1;
	 }

	 if (M141->hertzRate > *maxHertzRate)
	 {
		M141->hertzRate = *maxHertzRate;
	 }

	 M141->hertzRate = 1.0/((((1.0/(double)M141->hertzRate)/ tickTime))*tickTime);

	 goto GET_RATE;
  }
  else if (ch != 'Y' && ch != 'y')
  {
	 goto GET_RATE;
  }

  if (M141->hertzRate > *maxHertzRate)
  {
	 M141->hertzRate = *maxHertzRate;
  }

  return 1;
}

short CheckADC(unsigned int adc141)
{
  if (adc141 == 544 ||
		adc141 == 552 ||
		adc141 == 560 ||
		adc141 == 568 ||
		adc141 == 576 ||
		adc141 == 584 ||
		adc141 == 592 ||
		adc141 == 600 ||
		adc141 == 608 ||
		adc141 == 616 ||
		adc141 == 624)
  {
	 return 1;
  }
  return 0;
}

void    StandardDev(double *stDv,
						  double *rmsBits,
						  double *sdTot,
						  double *runningAvg,
						  double *sd,
						  double *voltage,
						  int    *sdCount,
						  int		 biPolar,
						  int		 sdGrp)
{//standard deviation and RMS resolution calculation
int i = 0, nnn;
double tempStDv, StDvRms;

  sd[*sdCount] = (*voltage);
  (*sdTot) += (*voltage);

  if ((*sdCount) >= sdGrp)
  {
	 (*stDv) = 0; (*sdTot) = (*sdTot) / sdGrp;

	 for (nnn = 1; nnn != sdGrp; nnn++)
	 {
		(*stDv) = (*stDv) + pow(((*sdTot) - sd[nnn]), 2);
	 }

	 (*stDv) = sqrt((*stDv) / sdGrp);

	 if (biPolar == 0x80)
	 {
		StDvRms = (*stDv) / 9.536743e-3;
	 }
	 else
	 {
		StDvRms = (*stDv) / 4.768372e-3;
	 }

	 tempStDv = StDvRms;

CALCNEXT:

	 if (tempStDv >= 1)
	 {
		i = i + 1;
		tempStDv = tempStDv / 2;
		goto CALCNEXT;
	 }

	 (*rmsBits) = 20 - i;

	 if ((tempStDv < .75) && (i != 0))
	 {
		(*rmsBits) = (*rmsBits) + .5;
	 }

	 (*runningAvg) = (*sdTot);
	 (*sdTot) = 0; (*sdCount) = 1;
  }
  else
  {
	 (*sdCount)++;
  }

}

void WaitSettle(T_M141 far *M141)	//if a new channel is selected or a
												//calibration is performed, wait time
												//for channel voltage to settle
{
DWORD settle, start;

  if (M141->calTime >= 5000)
  {
	 settle = 2270;
  }
  else if ((M141->calTime > 1500) && (M141->calTime < 5000))
  {
	 settle = 568;
  }
  else if (M141->calTime <= 1500)
  {
	 settle = 142;
  }

  gotoxy(1,24);
  printf("                        waiting settling time...                        ");
  start = GetTickCount() + settle;
  while (start > GetTickCount()); 				//wait settling time
  gotoxy(1,24);
  printf("                                              ");
}

